---------------------------------------------------------------------------------------------
QHVERSION = "QuickHeal v2.2.3.1"
---------------------------------------------------------------------------------------------
-- Author
--
--	WakeZero (Maleena on Kel'Thuzad | Force [Alliance])
-- 
-- Description
--
--	This mod allows a healer to quickly cast spells with a single mouse click.
--
-- Example
--
--	When enabled, damage taken bars will appear beside each player and party
--	unit frame. These bars increase with the amount of damage a player in the
--	party recieves, changing colors from green to yellow to red upon death.
--
--	By opening the options menu (/QH or assigned keybinding) the user may
--	choose spells from their spellbook to cast upon the corresponding party
--	member when the bars are clicked. Currently the Left, Middle and Right
--	mouse buttons may be assigned to.
--
-- TODO
--
--	* Localize for other languages
--	* Add class specific bindings and options
--	* Add raid frames
--	* Clean up logic o.O
--
-- Changelog
--
--	v2.2.3.1 [2007-10-29] : - Added Option Tabs
--
--				- Added Shift, Alt, and Control bindings
--
--				- Added Damage Bar Transparency Setting
--
--				- Added Damage Bar Value Display
--
--				- Added Resurrection to Left Click When Party Member Dies
--
--
--	v2.2.3.0 [2007-10-25] : - First Release
--
-- Shoutouts
--
--	Jesus Christ - For joy, life and eternal love (and the whole forgiveness of sins bit)
--
--	Curse (wow.curse.com) - For hosting my mod and many others.
--
--	WoWWiki (www.wowwiki.com) - Greatest resource and API for WoW mod developers.
--
--	Force (www.forceguild.net) - Grats on Illidan, keep kicking ass!
--
--	Copper06 - For suggesting the damage bar value display.
--
---------------------------------------------------------------------------------------------
-- Variables
---------------------------------------------------------------------------------------------
-- Player Profile
QHProfile = {};
PlayerName = nil;
QHplayerclass = nil;

-- Key Bindings
BINDING_HEADER_QuickHeal = "QuickHeal";

-- QuickHeal Options Variables
QHrunning = nil;
QHtab = "OPTIONS";
QHplayerleft = nil;
QHplayerleftshift = nil;
QHplayerleftctrl = nil;
QHplayerleftalt = nil;
QHplayermiddle = nil;
QHplayermiddleshift = nil;
QHplayermiddlectrl = nil;
QHplayermiddlealt = nil;
QHplayerright = nil;
QHplayerrightshift = nil;
QHplayerrightctrl = nil;
QHplayerrightalt = nil;
QHpartyleft = nil;
QHpartyleftshift = nil;
QHpartyleftctrl = nil;
QHpartyleftalt = nil;
QHpartymiddle = nil;
QHpartymiddleshift = nil;
QHpartymiddlectrl = nil;
QHpartymiddlealt = nil;
QHpartyright = nil;
QHpartyrightshift = nil;
QHpartyrightctrl = nil;
QHpartyrightalt = nil;
QHtransparency = 0;
QHresurrection = nil;
QHdamagebartext = nil;

-- TargetFrame Position Variables
QHtfpoint = nil;
QHtfrelativeTo = nil;
QHtfrelativePoint = nil;
QHtfxofs = nil;
QHtfyofs = nil;

-- State Headers
stateheaders = {};
stateheaders[1] = CreateFrame("Frame", "StateHeader", nil, "SecureStateHeaderTemplate");
for i=2,5 do
	stateheaders[i] = CreateFrame("Frame", "StateHeader", nil, "SecureStateHeaderTemplate");
end

---------------------------------------------------------------------------------------------
-- Setup/Profile Functions
---------------------------------------------------------------------------------------------
function QH_OnLoad()

	-- Setup Slash Commands
	QH_SetupCommands();
	
	-- Set Events
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PLAYER_LEAVE_COMBAT");
	this:RegisterEvent("PLAYER_REGEN_ENABLED");
end

function QH_SetupCommands()

	-- Toggles QH on and off
	SlashCmdList["QH"] = QH_ShowOptions;
	SLASH_QH1 = "/QH";
	SLASH_QH2 = "/QuickHeal";
end

function QH_Setup()
	
	-- Initialize Profile Variables
	QH_InitializeProfile();

	-- Store Default TargetFrame Position
	QHtfpoint,QHtfrelativeTo,QHtfrelativePoint,QHtfxofs,QHtfyofs = getglobal("TargetFrame"):GetPoint();

	-- Store Player Class
	local playerClass, englishClass = UnitClass("player");
	QHplayerclass = englishClass;
		
	-- Setup UI
	QH_SetupUI();
end

function QH_SetupUI()

	-- Update Status Bars
	QH_UpdateStatusBars("player");
	QH_UpdateStatusBars("party1");
	QH_UpdateStatusBars("party2");
	QH_UpdateStatusBars("party3");
	QH_UpdateStatusBars("party4");

	-- Update Buttons
	QH_SetupUnitButton("player");
	QH_SetupUnitButton("party1");
	QH_SetupUnitButton("party2");
	QH_SetupUnitButton("party3");
	QH_SetupUnitButton("party4");

	-- Adjust Target Frame To Accomodate For Player Status Bar
	getglobal("TargetFrame"):ClearAllPoints();
	if(QHrunning) then

		getglobal("TargetFrame"):SetPoint(QHtfpoint, QHtfrelativeTo, QHtfrelativePoint, QHtfxofs + 100, QHtfyofs);
	else
		getglobal("TargetFrame"):SetPoint(QHtfpoint, QHtfrelativeTo, QHtfrelativePoint, QHtfxofs, QHtfyofs);
	end
end

function QH_InitializeProfile()

	if ( UnitName("player") ) then
		PlayerName = UnitName("player").."|"..QH_Trim(GetCVar("realmName"));
		QH_LoadSettings();
	end

	QH_Report();
end

function QH_LoadSettings()

	if ( QHProfile[PlayerName] == nil ) then
		QHProfile[PlayerName] = {};
		QHProfile[PlayerName]["QHrunning"] = 'true';
		QHProfile[PlayerName]["QHtab"] = "OPTIONS";
		QHProfile[PlayerName]["QHplayerleft"] = nil;
		QHProfile[PlayerName]["QHplayerleftshift"] = nil;
		QHProfile[PlayerName]["QHplayerleftctrl"] = nil;
		QHProfile[PlayerName]["QHplayerleftalt"] = nil;
		QHProfile[PlayerName]["QHplayermiddle"] = nil;
		QHProfile[PlayerName]["QHplayermiddleshift"] = nil;
		QHProfile[PlayerName]["QHplayermiddlectrl"] = nil;
		QHProfile[PlayerName]["QHplayermiddlealt"] = nil;
		QHProfile[PlayerName]["QHplayerright"] = nil;
		QHProfile[PlayerName]["QHplayerrightshift"] = nil;
		QHProfile[PlayerName]["QHplayerrightctrl"] = nil;
		QHProfile[PlayerName]["QHplayerrightalt"] = nil;
		QHProfile[PlayerName]["QHpartyleft"] = nil;
		QHProfile[PlayerName]["QHpartyleftshift"] = nil;
		QHProfile[PlayerName]["QHpartyleftctrl"] = nil;
		QHProfile[PlayerName]["QHpartyleftalt"] = nil;
		QHProfile[PlayerName]["QHpartymiddle"] = nil;
		QHProfile[PlayerName]["QHpartymiddleshift"] = nil;
		QHProfile[PlayerName]["QHpartymiddlectrl"] = nil;
		QHProfile[PlayerName]["QHpartymiddlealt"] = nil;
		QHProfile[PlayerName]["QHpartyright"] = nil;
		QHProfile[PlayerName]["QHpartyrightshift"] = nil;
		QHProfile[PlayerName]["QHpartyrightctrl"] = nil;
		QHProfile[PlayerName]["QHpartyrightalt"] = nil;
		QHProfile[PlayerName]["QHtransparency"] = 0;
		QHProfile[PlayerName]["QHresurrection"] = 'true';
		QHProfile[PlayerName]["QHdamagebartext"] = 'true';
	end

	QHrunning = QHProfile[PlayerName]["QHrunning"];
	QHtab = QHProfile[PlayerName]["QHtab"];
	QHplayerleft = QHProfile[PlayerName]["QHplayerleft"];
	QHplayerleftshift = QHProfile[PlayerName]["QHplayerleftshift"];
	QHplayerleftctrl = QHProfile[PlayerName]["QHplayerleftctrl"];
	QHplayerleftalt = QHProfile[PlayerName]["QHplayerleftalt"];
	QHplayermiddle = QHProfile[PlayerName]["QHplayermiddle"];
	QHplayermiddleshift = QHProfile[PlayerName]["QHplayermiddleshift"];
	QHplayermiddlectrl = QHProfile[PlayerName]["QHplayermiddlectrl"];
	QHplayermiddlealt = QHProfile[PlayerName]["QHplayermiddlealt"];
	QHplayerright = QHProfile[PlayerName]["QHplayerright"];
	QHplayerrightshift = QHProfile[PlayerName]["QHplayerrightshift"];
	QHplayerrightctrl = QHProfile[PlayerName]["QHplayerrightctrl"];
	QHplayerrightalt = QHProfile[PlayerName]["QHplayerrightalt"];
	QHpartyleft = QHProfile[PlayerName]["QHpartyleft"];
	QHpartyleftshift = QHProfile[PlayerName]["QHpartyleftshift"];
	QHpartyleftctrl = QHProfile[PlayerName]["QHpartyleftctrl"];
	QHpartyleftalt = QHProfile[PlayerName]["QHpartyleftalt"];
	QHpartymiddle = QHProfile[PlayerName]["QHpartymiddle"];
	QHpartymiddleshift = QHProfile[PlayerName]["QHpartymiddleshift"];
	QHpartymiddlectrl = QHProfile[PlayerName]["QHpartymiddlectrl"];
	QHpartymiddlealt = QHProfile[PlayerName]["QHpartymiddlealt"];
	QHpartyright = QHProfile[PlayerName]["QHpartyright"];
	QHpartyrightshift = QHProfile[PlayerName]["QHpartyrightshift"];
	QHpartyrightctrl = QHProfile[PlayerName]["QHpartyrightctrl"];
	QHpartyrightalt = QHProfile[PlayerName]["QHpartyrightalt"];
	QHtransparency = QHProfile[PlayerName]["QHtransparency"];
	QHresurrection = QHProfile[PlayerName]["QHresurrection"];
	QHdamagebartext = QHProfile[PlayerName]["QHdamagebartext"];

	if(QHtransparency == nil) then QHtransparency = 0; end
end

function QH_ResetSettings()

	QHProfile[PlayerName] = nil;
	QH_LoadSettings();
	QH_UpdateOptions();
	QH_SetupUI();
end

function QH_SaveVar(varname, value)

	if ( PlayerName ) then
		if ( QHProfile[PlayerName] ) then
			QHProfile[PlayerName][varname] = value;
		end
	end
end

---------------------------------------------------------------------------------------------
-- Event Handlers
---------------------------------------------------------------------------------------------
function QH_OnEvent()

        if (event == "VARIABLES_LOADED") then

		QH_Setup();

	elseif(event == "PLAYER_LEAVE_COMBAT" or event == "PLAYER_REGEN_ENABLED") then

		QH_SetupUI();
	end
end

---------------------------------------------------------------------------------------------
-- Options GUI Functions
---------------------------------------------------------------------------------------------
function QH_ShowOptions()

	if (QHOptionsFrame:IsVisible()) then
		HideUIPanel(QHOptionsFrame);
		HideUIPanel(QHOptionsFrameTabs);
	else
		QH_ChangeTabs();
		ShowUIPanel(QHOptionsFrame);
		ShowUIPanel(QHOptionsFrameTabs);
	end
end

function QH_HideAllOptions()

	HideUIPanel(QHBOXONOFF);
	HideUIPanel(QHBOXBARTEXT);
	HideUIPanel(QHBOXRESURRECTION);
	HideUIPanel(QHTransparencySlider);

	HideUIPanel(QHleftdropdown);	
	HideUIPanel(QHleftshiftdropdown);
	HideUIPanel(QHleftctrldropdown);
	HideUIPanel(QHleftaltdropdown);

	HideUIPanel(QHmiddledropdown);
	HideUIPanel(QHmiddleshiftdropdown);
	HideUIPanel(QHmiddlectrldropdown);
	HideUIPanel(QHmiddlealtdropdown);

	HideUIPanel(QHrightdropdown);
	HideUIPanel(QHrightshiftdropdown);
	HideUIPanel(QHrightctrldropdown);
	HideUIPanel(QHrightaltdropdown);
end

function QH_UpdateOptions()

	QH_HideAllOptions();

	if(QHtab == nil or QHtab == "OPTIONS") then

		QHTABLABEL:SetText(QH_LOCALE_OPTIONSTAB);

		ShowUIPanel(QHBOXONOFF);
		ShowUIPanel(QHBOXBARTEXT);
		ShowUIPanel(QHBOXRESURRECTION);
		ShowUIPanel(QHTransparencySlider);
		
		-- Setup CheckBoxes
		QHBOXONOFF:SetChecked(QHrunning);
		QHBOXBARTEXT:SetChecked(QHdamagebartext);
		QHBOXRESURRECTION:SetChecked(QHresurrection);

		-- Setup Transparency Slider
		QHTransparencySlider:SetValue(QHtransparency);

	elseif(QHtab == "PLAYER") then

		QHTABLABEL:SetText(QH_LOCALE_PLAYERTAB);

		-- Setup Dropdown Menus
		QH_SetupDropdown(QHleftdropdown, QHplayerleft);
		QH_SetupDropdown(QHleftshiftdropdown, QHplayerleftshift);
		QH_SetupDropdown(QHleftctrldropdown, QHplayerleftctrl);
		QH_SetupDropdown(QHleftaltdropdown, QHplayerleftalt);

		QH_SetupDropdown(QHmiddledropdown, QHplayermiddle);
		QH_SetupDropdown(QHmiddleshiftdropdown, QHplayermiddleshift);
		QH_SetupDropdown(QHmiddlectrldropdown, QHplayermiddlectrl);
		QH_SetupDropdown(QHmiddlealtdropdown, QHplayermiddlealt);

		QH_SetupDropdown(QHrightdropdown, QHplayerright);
		QH_SetupDropdown(QHrightshiftdropdown, QHplayerrightshift);
		QH_SetupDropdown(QHrightctrldropdown, QHplayerrightctrl);
		QH_SetupDropdown(QHrightaltdropdown, QHplayerrightalt);

	elseif(QHtab == "PARTY") then

		QHTABLABEL:SetText(QH_LOCALE_PARTYTAB);

		-- Setup Dropdown Menus
		QH_SetupDropdown(QHleftdropdown, QHpartyleft);
		QH_SetupDropdown(QHleftshiftdropdown, QHpartyleftshift);
		QH_SetupDropdown(QHleftctrldropdown, QHpartyleftctrl);
		QH_SetupDropdown(QHleftaltdropdown, QHpartyleftalt);

		QH_SetupDropdown(QHmiddledropdown, QHpartymiddle);
		QH_SetupDropdown(QHmiddleshiftdropdown, QHpartymiddleshift);
		QH_SetupDropdown(QHmiddlectrldropdown, QHpartymiddlectrl);
		QH_SetupDropdown(QHmiddlealtdropdown, QHpartymiddlealt);

		QH_SetupDropdown(QHrightdropdown, QHpartyright);
		QH_SetupDropdown(QHrightshiftdropdown, QHpartyrightshift);
		QH_SetupDropdown(QHrightctrldropdown, QHpartyrightctrl);
		QH_SetupDropdown(QHrightaltdropdown, QHpartyrightalt);

	end
end

function QH_SetupDropdown(dropdown, value)

	ShowUIPanel(dropdown);
	UIDropDownMenu_Initialize(dropdown, QH_InitializeDropdown);
	UIDropDownMenu_SetWidth(150, dropdown);
	UIDropDownMenu_SetText("", dropdown);
	UIDropDownMenu_SetSelectedValue(dropdown, value);

end

function QH_InitializeDropdown()

	local spells = "";
	info = UIDropDownMenu_CreateInfo();

	-- Populate list with available spells in spellbook
	local i=1; while true do

   		local spellname, spellrank = GetSpellName(i, BOOKTYPE_SPELL);

		-- Exit loop if nil
   		if not spellname then
		do break end
   		end

		-- Add Spell to Menu
		if((IsSpellInRange(spellname, "player") or IsSpellInRange(spellname, "party")) and not IsPassiveSpell(i, BOOKTYPE_SPELL) and strfind(spells, spellname) == nil) then

			spells = spells .. spellname .. ";";
		end
		
   		i=i+1;
	end

	-- Sort Spell List
	slist = {strsplit(";", spells)};
	sort(slist);

	-- Add Spells to Dropdown
	for i=1,#(slist) do
		
		info = {};	
		info.text = slist[i];--spellname;
		info.value = slist[i];--spellname;
		info.owner = getglobal(UIDROPDOWNMENU_INIT_MENU);
		info.func = QHDropdown_OnClick;
			
		UIDropDownMenu_AddButton(info);
	end
end

function QH_ChangeTabs(tab)

	if(tab ~= nil) then
		QHtab = tab;
		QH_SaveVar("QHtab", QHtab);
	end
	
	QHPartyTab:SetTextColor(1,1,0,1);
	QHPlayerTab:SetTextColor(1,1,0,1);
	QHOptionsTab:SetTextColor(1,1,0,1);

	if(QHtab == "OPTIONS") then

		QHOptionsTab:SetTextColor(1,1,1,1);

	elseif(QHtab == "PLAYER") then

		QHPlayerTab:SetTextColor(1,1,1,1);

	elseif(QHtab == "PARTY") then

		QHPartyTab:SetTextColor(1,1,1,1);

	end

	QH_UpdateOptions();
end

function QHDropdown_OnClick()

	-- Set Dropdown Value	
	UIDropDownMenu_SetSelectedValue(this.owner, this.value);
	
	-- Save Settings
	local var = gsub(this.owner:GetName(), "(dropdown)", "");
	if(QHtab == "PLAYER") then

		var = gsub(var, "(QH)", "QHplayer");
	elseif(QHtab == "PARTY") then

		var = gsub(var, "(QH)", "QHparty");
	end
	setglobal(var, this.value);
	QH_SaveVar(var, this.value);

	-- Update Unit Buttons
	QH_SetupUI();
end

function QHOptionsCheckButton_OnLoad()

	-- Setup Check Boxes
	if(this == QHBOXONOFF) then
		getglobal(this:GetName().."Text"):SetText(QH_LOCALE_RUNNING);
	elseif(this == QHBOXBARTEXT) then
		getglobal(this:GetName().."Text"):SetText(QH_LOCALE_BARTEXT);
	elseif(this == QHBOXRESURRECTION) then
		getglobal(this:GetName().."Text"):SetText(QH_LOCALE_RESURRECTION);
	end

end

function QHOptionsCheckButton_OnClick()

	-- Check which box was clicked
	if(this == QHBOXONOFF) then

		QHrunning = this:GetChecked();
		QH_SaveVar("QHrunning", QHrunning);

	elseif(this == QHBOXBARTEXT) then

		QHdamagebartext = this:GetChecked();
		QH_SaveVar("QHdamagebartext", QHdamagebartext);

	elseif(this == QHBOXRESURRECTION) then

		QHresurrection = this:GetChecked();
		QH_SaveVar("QHresurrection", QHresurrection);
		
	end

	-- Update Unit Buttons
	QH_SetupUI();
end

---------------------------------------------------------------------------------------------
-- Spell Functions
---------------------------------------------------------------------------------------------
function QH_SetupUnitButton(unit)
	
	local button = this;
	if(unit ~= nil) then
		button = getglobal("QHButton" .. unit);
	end
	
	if(QHrunning) then

		button:Enable();
		local id = button:GetID();
		button:RegisterForClicks('LeftButtonUp', 'RightButtonUp', 'MiddleButtonUp', 'Button4Up', 'Button5Up');

		local resurrection = QH_LOCALE_PRIESTRES;
		if(QHplayerclass == "DRUID") then
			resurrection = QH_LOCALE_DRUIDRES;
		elseif(QHplayerclass == "PALADIN") then
			resurrection = QH_LOCALE_PALADINRES;
		elseif(QHplayerclass == "SHAMAN") then
			resurrection = QH_LOCALE_SHAMANRES;
		end

		if(unit == "player") then

			button:SetAttribute("unit", "player");

			-- LEFT CLICK
			button:SetAttribute("*helpbutton1", "playerleft");
			button:SetAttribute("*type-playerleft", "spell");
			button:SetAttribute("spell-playerleft", QHplayerleft);
			button:SetAttribute("shift-spell-playerleft", QHplayerleftshift);
			button:SetAttribute("ctrl-spell-playerleft", QHplayerleftctrl);
			button:SetAttribute("alt-spell-playerleft", QHplayerleftalt);

			-- RIGHT CLICK
			button:SetAttribute("*helpbutton2", "playerright");
			button:SetAttribute("*type-playerright", "spell");
			button:SetAttribute("spell-playerright", QHplayerright);
			button:SetAttribute("shift-spell-playerright", QHplayerrightshift);
			button:SetAttribute("ctrl-spell-playerright", QHplayerrightctrl);
			button:SetAttribute("alt-spell-playerright", QHplayerrightalt);

			-- MIDDLE CLICK
			button:SetAttribute("*helpbutton3", "playermiddle");
			button:SetAttribute("*type-playermiddle", "spell");
			button:SetAttribute("spell-playermiddle", QHplayermiddle);
			button:SetAttribute("shift-spell-playermiddle", QHplayermiddleshift);
			button:SetAttribute("ctrl-spell-playermiddle", QHplayermiddlectrl);
			button:SetAttribute("alt-spell-playermiddle", QHplayermiddlealt);
		
		else

			button:SetAttribute("unit", "party"..id-1);

			if(QHresurrection) then

				-- PREVENT
				if(QHpartyleft == nil) then QHpartyleft = ""; end
				if(QHpartyleftshift == nil) then QHpartyleftshift = ""; end
				if(QHpartyleftctrl == nil) then QHpartyleftctrl = ""; end
				if(QHpartyleftalt == nil) then QHpartyleftalt = ""; end

				-- LEFT CLICK
				button:SetAttribute("*helpbutton1", "partyleft");
				button:SetAttribute("*type-partyleft", "macro");
				button:SetAttribute("macrotext-partyleft", "/cast [target=" .. unit .. ", dead] " .. resurrection .. "; [target=" .. unit .. "] " .. QHpartyleft .. ";");
				button:SetAttribute("shift-macrotext-partyleft", "/cast [target=" .. unit .. ", dead] " .. resurrection .. "; [target=" .. unit .. "] " .. QHpartyleftshift .. ";");
				button:SetAttribute("ctrl-macrotext-partyleft", "/cast [target=" .. unit .. ", dead] " .. resurrection .. "; [target=" .. unit .. "] " .. QHpartyleftctrl .. ";");
				button:SetAttribute("alt-macrotext-partyleft", "/cast [target=" .. unit .. ", dead] " .. resurrection .. "; [target=" .. unit .. "] " .. QHpartyleftalt .. ";");
			
			else
				-- LEFT CLICK
				button:SetAttribute("*helpbutton1", "partyleft");
				button:SetAttribute("*type-partyleft", "spell");
				button:SetAttribute("spell-partyleft", QHpartyleft);
				button:SetAttribute("shift-spell-partyleft", QHpartyleftshift);
				button:SetAttribute("ctrl-spell-partyleft", QHpartyleftctrl);
				button:SetAttribute("alt-spell-partyleft", QHpartyleftalt);
			end

			-- RIGHT CLICK
			button:SetAttribute("*helpbutton2", "partyright");
			button:SetAttribute("*type-partyright", "spell");
			button:SetAttribute("spell-partyright", QHpartyright);
			button:SetAttribute("shift-spell-partyright", QHpartyrightshift);
			button:SetAttribute("ctrl-spell-partyright", QHpartyrightctrl);
			button:SetAttribute("alt-spell-partyright", QHpartyrightalt);

			-- MIDDLE CLICK
			button:SetAttribute("*helpbutton3", "partymiddle");
			button:SetAttribute("*type-partymiddle", "spell");
			button:SetAttribute("spell-partymiddle", QHpartymiddle);
			button:SetAttribute("shift-spell-partymiddle", QHpartymiddleshift);
			button:SetAttribute("ctrl-spell-partymiddle", QHpartymiddlectrl);
			button:SetAttribute("alt-spell-partymiddle", QHpartymiddlealt);
		end
	else

		button:Disable();
	end
end

---------------------------------------------------------------------------------------------
-- UI Functions
---------------------------------------------------------------------------------------------
function QH_UpdateStatusBars(unit)
	
	local bar, button, label;
	if(unit ~= nil) then
		bar = getglobal("QHBar"..unit);
		button = getglobal("QHButton"..unit);
		label = getglobal("QHBar"..unit.."text");
	end
	
	if(QHrunning and bar ~= nil) then

		bar:Show();

		if(unit == "player" or unit == "party1" or unit == "party2" or unit == "party3" or unit == "party4") then

			local health = UnitHealth(unit);
			
			local button = getglobal("QHButton"..unit);
			local healthmax = UnitHealthMax(unit);
			local healthdefecit = healthmax - health;
	
			bar:SetMinMaxValues(0, healthmax);

			if(QHdamagebartext) then

				local text = button:CreateFontString("QHButton"..unit.."text");
				text:SetAllPoints(button);
				text:SetJustifyH('LEFT');
				text:SetFontObject('GameFontNormal');
				button:SetFontString(text);
				button:SetTextColor(1, 1, 1, QHtransparency / -100 + 1);
				button:SetHighlightTextColor(1, 1, 1, QHtransparency / -100 + 1);

				if(healthdefecit > 0) then
					button:SetText("-" .. healthdefecit);
				else
					button:SetText("");
				end
			else
				button:SetText("");
			end
			
			if(UnitIsDeadOrGhost(unit)) then

				bar:SetValue(healthmax);
				bar:SetStatusBarColor(1, 0, 0, QHtransparency / -100 + 1);
				stateheaders[button:GetID()]:SetAttribute("state", "2"); --Only works out of combat

			elseif(not UnitIsConnected(unit)) then

				bar:SetValue(0);
				bar:SetStatusBarColor(0, 0, 0, QHtransparency / -100 + 1);
				stateheaders[button:GetID()]:SetAttribute("state", "1"); --Only works out of combat

			elseif(health and healthmax) then

				bar:SetValue(healthmax - health);

				if((health / healthmax) < 0.66) then
					bar:SetStatusBarColor(1, 1, 0, QHtransparency / -100 + 1);
				else
					bar:SetStatusBarColor(0, 1, 0, QHtransparency / -100 + 1);
				end

				stateheaders[button:GetID()]:SetAttribute("state", "1"); --Only works out of combat
			end
		end

	elseif(bar ~=nil) then

		bar:Hide();
	end
end

---------------------------------------------------------------------------------------------
-- Utility Functions
---------------------------------------------------------------------------------------------
--TODO: Add color
function QH_Print(msg, red, green, blue)
	DEFAULT_CHAT_FRAME:AddMessage(msg);
end

function QH_Trim(s)
	return (string.gsub(s, "^%s*(.-)%s*$", "%1"));
end

---------------------------------------------------------------------------------------------
-- Slash Command Functions
---------------------------------------------------------------------------------------------
function QH_Report()
	if(QHrunning) then
		QH_Print("QuickHeal is |cFF00FF00ON".."|cFFFFFFFF ["..QH_LOCALE_HELP.."]");
	else
		QH_Print("QuickHeal is |cFFFF0000OFF".."|cFFFFFFFF ["..QH_LOCALE_HELP.."]");
	end
end

function QH_Help()

	QH_Print("|cFFFFFF00"..QH_LOCALE_HELP);
end

function QH_ToggleRun()

	if(QHrunning) then
		QHrunning = nil;
		QH_Print("QuickHeal is now |cFFFF0000OFF".."|cFFFFFFFF ["..QH_LOCALE_HELP.."]");
	else
		QHrunning = 'true';
		QH_Print("QuickHeal is now |cFF00FF00ON".."|cFFFFFFFF ["..QH_LOCALE_HELP.."]");
	end
end